@@ -16,8 +16,20 @@
 /* Define if ggUSleep is a #define in ggi/system.h */
 #undef GG_USLEEP_DEFINED
 
+/* Define if ggCurTime is a #define in ggi/system.h */
+#undef GG_CURTIME_DEFINED
+
+/* Define if sys/ioctl.h defines TIOCMSET */
+#undef HAVE_TIOCMSET
+
 /* Undefine to disable run-time debugging */
 #define DEBUG
 
 /* Define if your libc has a __malloc_hook */
 #undef HAVE_MALLOC_HOOK
+
+/*  Define if you have the nanosleep function.  */
+#undef HAVE_NANOSLEEP
+
+/*  Define if you have the sched_yield function.  */
+#undef HAVE_SCHED_YIELD
