@@ -1,4 +1,4 @@
-/* $Id: conf.c,v 1.6 1999/03/09 17:35:03 marcus Exp $
+/* $Id: conf.c,v 1.1.1.1 2001/05/12 23:00:04 cegger Exp $
 ******************************************************************************
   
    LibGG - Configuration handling
@@ -66,8 +66,7 @@ _ggDoLoad(const char *file, gg_dlconfig **list, int incl, char *oldroot)
 		root[2047] = '\0';
 	}
 
-	while (!feof(inf)) {
-		fgets(line, 2048, inf);
+	while (fgets(line, 2048, inf)) {
 		cp = line;
 
 		while (isspace((int)*cp)) cp++;
@@ -119,7 +118,9 @@ _ggDoLoad(const char *file, gg_dlconfig **list, int incl, char *oldroot)
 		*(bp) = '\0';
 		if (buff[0] != CHAR_DIRDELIM) {
 			if ((fname = (char *) malloc(strlen(buff)
-						     +strlen(root)+2))
+						     +strlen(root)
+						     +strlen(STRING_DIRDELIM)
+						     +1))
 			    == NULL) {
 				return GGI_ENOMEM;
 			}
@@ -230,3 +231,36 @@ const char *ggMatchConfig(const void *conf, const char *name,const char *ver)
 
 	return (best == NULL) ? NULL : best->filename;
 }
+
+int ggConfigExpandAlias(void *confhandle, const char *list_in, 
+	char *list_out, size_t outmax) {
+	
+	char *current,*newcur;
+	const char *match;
+	char buffer[1024];
+	int loopcnt=0;
+	int matchlen,outremain;
+
+	if (outmax>0) *list_out='\0';
+	if (strlen(list_in)+1>outmax) return GGI_ENOMEM;
+	strcpy(list_out,list_in);
+	outremain=outmax-strlen(list_out)-1;
+
+	current=list_out;
+	while(*current) {
+		if (*current==':') {current++;continue;}
+		newcur=ggParseTarget(current,buffer,sizeof(buffer));
+		if (NULL==newcur) return GGI_EARGINVAL;
+		match=ggMatchConfig(confhandle, buffer, NULL);
+		if (match&&match[0]=='*') {
+			if (loopcnt++>100) return GGI_EARGINVAL; /* FIXME ELOOP */
+			matchlen=strlen(match+2);
+			if (outremain+newcur-current-matchlen>0) {
+				 memmove(current+matchlen,newcur,strlen(newcur)+1);
+				 memcpy(current,match+2,matchlen);
+				 outremain+=newcur-current-matchlen-1;
+			} else return GGI_ENOMEM;
+		} else current=newcur;
+	}
+	return 0;
+}
