@@ -1,4 +1,4 @@
-/* $Id: dlptlock.c,v 1.5 1999/02/11 00:32:50 marcus Exp $
+/* $Id: dlptlock.c,v 1.1.1.1 2001/05/12 23:00:04 cegger Exp $
 ******************************************************************************
 
    LibGG - Mutex implementation using pthreads. 
@@ -62,7 +62,7 @@ void  *ggLockCreate(void)
 
 int ggLockDestroy(void *lock) {
 	if (my_destroy((pthread_mutex_t*)lock) != 0) {
-		return GGI_ELOCKBUSY;
+		return GGI_EBUSY;
 	}
 	free(lock);
 	return 0;
@@ -87,7 +87,7 @@ int ggTryLock(void *lock) {
 	switch (my_trylock((pthread_mutex_t*)lock)) {
 	case 0:	return 0;
 
-	case EBUSY: return GGI_ELOCKBUSY;
+	case EBUSY: return GGI_EBUSY;
 	}
 	return GGI_EUNKNOWN;
 }
@@ -96,7 +96,9 @@ void _ggExitLocks(void)
 {
 
 /*	ARGH ! LibC seems to see the symbols from the dlopened pthreads and
-	causes a segfault, when it is gone ... AAAAAHHH ! [becka] */
+	causes a segfault, when it is gone ... AAAAAHHH ! [becka] 
+	This is why this code is disabled, though it would be "the right
+	thing to do". */
 #if 0
 	if (isup) dlclose(handle);
 #endif
