@@ -1,4 +1,4 @@
-/* $Id: plat.h,v 1.9 1999/03/02 02:29:43 marcus Exp $
+/* $Id: plat.h,v 1.2 2001/06/26 02:16:22 fortinj Exp $
 ******************************************************************************
 
    LibGG internal definitions
@@ -28,64 +28,57 @@
 #ifndef _GG_INTERNAL_PLAT_H
 #define _GG_INTERNAL_PLAT_H
 
+#include <ggi/system.h>
 #include "config.h"
 
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+# include <unistd.h>
 #endif
 
 /* Mutex for protecting LibGG global data */
 extern void	*_gg_global_mutex;
 
 #if defined(__GNUC__) && !defined(inline)
-#define inline __inline__
+# define inline __inline__
 #endif
 
-#if  defined(__CYGWIN32__) \
-  || defined(__MINGW32__)
-#ifndef __WIN32__
-#define __WIN32__
-#endif
-#endif
-
-#ifdef __WIN32__
-#define CHAR_DIRDELIM '\\'
-#define STRING_DIRDELIM "\\"
+#if defined(__WIN32__) && !defined(__CYGWIN__)
+# define STRING_DIRDELIM "\\"
+# define STRING_DIRDELIM "\\"
 #else
-#define CHAR_DIRDELIM '/'
-#define STRING_DIRDELIM "/"
+# define CHAR_DIRDELIM '/'
+# define STRING_DIRDELIM "/"
 #endif
 
-#ifdef _WIN32
-#include <windows.h> 
-#define GGOPENLIB(filename)	LoadLibrary((filename))
-#define GGOPENLIBGLOBAL(filename) GGOPENLIB(filename)
-#define GGGETSYM(handle,sym)	GetProcAddress((handle),(sym))
-#define GGCLOSELIB(handle)	FreeLibrary((handle))
-#define GGDLERROR()		NULL
-#else /* _WIN32 */
-#include <dlfcn.h>
-#ifdef RTLD_LAZY
-#define GGOPENLIB(filename)		dlopen((filename), RTLD_LAZY)
-#ifdef RTLD_GLOBAL
-#define GGOPENLIBGLOBAL(filename)	dlopen((filename), RTLD_LAZY | RTLD_GLOBAL)
+#if defined(__WIN32__) && !defined(__CYGWIN__)
+# include <windows.h> 
+# define GGOPENLIB(filename)	LoadLibrary((filename))
+# define GGOPENLIBGLOBAL(filename) GGOPENLIB(filename)
+# define GGGETSYM(handle,sym)	GetProcAddress((handle),(sym))
+# define GGCLOSELIB(handle)	FreeLibrary((handle))
+# define GGDLERROR()		NULL
 #else
-#define GGOPENLIBGLOBAL(filename)	dlopen((filename), RTLD_LAZY)
-#endif /* RTLD_GLOBAL */
-#elif defined(DL_LAZY)
-#define GGOPENLIB(filename)		dlopen((filename), DL_LAZY)
-#define GGOPENLIBGLOBAL(filename)	dlopen((filename), DL_LAZY)
-#else
-#error This system has an unknown dlopen() call !!!
-#endif /* RTLD_LAZY */
-#define GGGETSYM(handle,sym)	dlsym((handle),(sym))
-#define GGCLOSELIB(handle)	dlclose((handle))
-#define GGDLERROR()		dlerror()
-#endif /* _WIN32 */
+# include <dlfcn.h>
+# ifdef RTLD_LAZY
+#  define GGOPENLIB(filename)		dlopen((filename), RTLD_LAZY)
+#  ifdef RTLD_GLOBAL
+#   define GGOPENLIBGLOBAL(filename)	dlopen((filename), RTLD_LAZY | RTLD_GLOBAL)
+#  else
+#   define GGOPENLIBGLOBAL(filename)	dlopen((filename), RTLD_LAZY)
+#  endif /* RTLD_GLOBAL */
+# elif defined(DL_LAZY)
+#  define GGOPENLIB(filename)		dlopen((filename), DL_LAZY)
+#  define GGOPENLIBGLOBAL(filename)	dlopen((filename), DL_LAZY)
+# else
+#  error This system has an unknown dlopen() call !!!
+# endif /* RTLD_LAZY */
+# define GGGETSYM(handle,sym)	dlsym((handle),(sym))
+# define GGCLOSELIB(handle)	dlclose((handle))
+# define GGDLERROR()		dlerror()
+#endif /* __WIN32__ */
 
-#if defined(SYMBOL_UNDERSCORE) || defined(__OpenBSD__)
-#define GG_SYMPREFIX		"_"
+#if defined(SYMBOL_UNDERSCORE) || defined(__OpenBSD__) || defined(__NetBSD__)
+# define GG_SYMPREFIX		"_"
 #endif
 
-#endif /* _GGI_INTERNAL_PLAT_H */
-
+#endif /* _GG_INTERNAL_PLAT_H */
