@@ -1,4 +1,4 @@
-/* $Id: gii.c,v 1.16 1999/04/22 20:26:10 marcus Exp $
+/* $Id: gii.c,v 1.4 2001/06/30 09:56:14 soyt Exp $
 ******************************************************************************
 
    Graphics library for GGI. General Input Interface.
@@ -36,6 +36,7 @@
 #include <ctype.h>
 
 #include <ggi/internal/gii.h>
+#include <ggi/internal/gii_debug.h>
 
 #ifdef HAVE_SIGNAL_H
 #include <signal.h>
@@ -50,6 +51,10 @@
 static uint32	_gii_origin_count = GII_MAXSUBLIBS;
 
 
+/* Macros */
+#define GII_EV_TIMESTAMP(ev)	ggCurTime(&((ev)->any.time))
+
+
 /*
 ******************************************************************************
  Internal standard handlers
@@ -87,7 +92,7 @@ _giiEvQueueSetup(void)
 {
 	gii_ev_queue *qp;
 	
-	GIIDPRINT_CORE("_giiEvQueueSetup() called\n");
+	GIID2(GIIDPRINT_CORE("_giiEvQueueSetup() called\n"));
 
 	qp = malloc(sizeof(gii_ev_queue));
 	if (qp == NULL) return NULL;
@@ -107,11 +112,12 @@ static void _giiEvQueueDestroy(gii_input *inp)
 	GIIDPRINT_CORE("_giiEvQueueDestroy(%p) called\n", inp);
 	
 	if (inp->queue) {
-		GIIDPRINT_CORE("Desttroying %p, %p\n",
+		GIIDPRINT_CORE("Destroying %p, %p\n",
 			       inp->queue, inp->queue->queues);
 		for (queue=0; queue < evLast; queue++) {
-			GIIDPRINT_CORE("_giiEvQueueDestroy going %d, %p\n",
-				       queue, inp->queue->queues[queue]);
+			GIID2(GIIDPRINT_CORE(
+				"_giiEvQueueDestroy going %d, %p\n",
+				queue, inp->queue->queues[queue]));
 			if (inp->queue->queues[queue]) {
 				free(inp->queue->queues[queue]);
 			}
@@ -270,10 +276,10 @@ _giiEvQueueRelease(gii_input *inp, gii_event *ev, gii_event_mask mask)
 	for (queue=0; queue < evLast; queue++) {
 		struct gii_ev_queue *qp_tmp;
 		
-		GIIDPRINT_EVENTS("queue = %p, queue->queues = %p, "
-				"queue->queues[queue] = %p\n",
-				inp->queue, inp->queue->queues,
-				inp->queue->queues[queue]);
+		GIID2(GIIDPRINT_EVENTS("queue = %p, queue->queues = %p, "
+				       "queue->queues[queue] = %p\n",
+				       inp->queue, inp->queue->queues,
+				       inp->queue->queues[queue]));
 		qp_tmp = inp->queue->queues[queue];
 		if (qp_tmp && qp_tmp->count && (evm & (1 << queue))) {
 			gii_event *e_tmp = (gii_event *)
@@ -283,7 +289,7 @@ _giiEvQueueRelease(gii_input *inp, gii_event *ev, gii_event_mask mask)
 			if (t_tmp.tv_sec < t_min.tv_sec || 
 			    (t_tmp.tv_sec == t_min.tv_sec &&
                              t_tmp.tv_usec < t_min.tv_usec)) {
-				GIIDPRINT_EVENTS("_giiEvQueueRelease: Plausible found.\n");
+				GIID2(GIIDPRINT_EVENTS("_giiEvQueueRelease: Plausible found.\n"));
 				qp = qp_tmp;
 				t_min = t_tmp;
 			}
@@ -384,7 +390,10 @@ int _giiEvQueueAdd(gii_input *inp, gii_event *ev)
 		qp = inp->queue->queues[ev->any.type];
 	} else {
 		qp = _giiEvQueueSetup();
-		if (qp == NULL) return GGI_ENOMEM;
+		if (qp == NULL) {
+			if (_gii_threadsafe) ggUnlock(inp->queue->mutex);
+			return GGI_ENOMEM;
+		}
 		inp->queue->queues[ev->any.type] = qp;
 	}
 
@@ -392,11 +401,19 @@ int _giiEvQueueAdd(gii_input *inp, gii_event *ev)
 			 ev->any.type, ev->size, qp->count);
 
 	ret = _giiAddEvent(qp, ev);
-	if (ret != 0) return ret;
+	if (ret != 0) {
+		if (_gii_threadsafe) ggUnlock(inp->queue->mutex);
+		return ret;
+	}
 
 	inp->queue->seen |= (1 << ev->any.type);
 
-	if (_gii_threadsafe) ggUnlock(inp->queue->mutex);
+	if (_gii_threadsafe) {
+		/* Notify any other threads which may be blocking on
+		   this input */
+		_giiAsyncNotify(inp);
+		ggUnlock(inp->queue->mutex);
+	}
 
 	return 0;
 }
@@ -414,7 +431,8 @@ int _giiSafeAdd(struct gii_input *inp, gii_event *ev)
 #endif
 	ggLock(_gii_safe_lock);
 
-	if (!inp->safequeue && (inp->safequeue = _giiEvQueueSetup()) == NULL) {
+	if (inp->safequeue == NULL && 
+	    (inp->safequeue = _giiEvQueueSetup()) == NULL) {
 		ret = GGI_ENOMEM;
 	} else {
 		_giiAddEvent(inp->safequeue, ev);
@@ -492,7 +510,6 @@ struct gii_input *_giiInputAlloc(void)
 	ret->mutex = NULL;
 	ret->next = ret->prev = ret;	/* Ring structure ... self<->self */
 	ret->dlhand = NULL;
-	ret->filters = NULL;
 	ret->devinfo = NULL;
 
 	ret->origin = _gii_origin_count++;
@@ -613,9 +630,10 @@ struct gii_input *giiOpen(const char *input,...)
 		GIIDPRINT_LIBS("giiOpen adding \"%s\", \"%s\", %p\n",
 			       target, args, argptr);
 		
-		/* Leave space for up to 256 subsystems ...
-		   I don't think we will wrap around - will we ? */
-		_gii_origin_count += GII_MAXSUBLIBS;	/* Start at next boundary */
+		/* Start at next boundary.  This leaves space for up 
+		 * to 256 subsystems ...  I don't think we will 
+		 * wrap around - will we ? */
+		_gii_origin_count += GII_MAXSUBLIBS; 
 		_gii_origin_count &= GII_MAINMASK;
 		_gii_origin_count &= ~GII_EV_ORIGIN_SENDEVENT;
 
@@ -676,6 +694,8 @@ int giiClose(struct gii_input *inp)
 	_giiEvQueueDestroy(inp);	/* This destroys _all_ queues ! */
 
 	do {
+		struct gii_input *prev;
+
 		curr->queue = NULL;	/* For better error catching. */
 
 		if (curr->GIIclose) {
@@ -686,9 +706,10 @@ int giiClose(struct gii_input *inp)
 			_giiCloseDL(curr->dlhand);
 			free(curr->dlhand);
 		}
+		prev = curr;
 		curr = curr->next;
-		_giiInputFree(curr->prev);
-	} while (curr!=inp);	/* looped through once. */
+		_giiInputFree(prev);
+	} while (curr != inp);	/* looped through once. */
 
 	return rc;
 }
@@ -703,11 +724,11 @@ struct gii_input *giiJoinInputs(struct gii_input *inp, struct gii_input *inp2)
 	 
 	GIIDPRINT_EVENTS("giiJoinInputs(%p, %p) called\n", inp, inp2);
 
-	if (!inp) {
+	if (inp == NULL) {
 		if (inp2) _giiUpdateCache(inp2);
 		return inp2;
 	}
-	if (!inp2) {
+	if (inp2 == NULL) {
 		if (inp) _giiUpdateCache(inp);
 		return inp;
 	}
@@ -755,6 +776,108 @@ struct gii_input *giiJoinInputs(struct gii_input *inp, struct gii_input *inp2)
 }
 
 
+int giiSplitInputs(struct gii_input *inp, struct gii_input **newhand,
+		   uint32 origin, uint32 flags)
+{
+	struct gii_input *idx;
+
+	flags = 0; /* Silence, GCC!  Flags is reserved for later use 
+		      when asking an input driver-lib to split off an 
+		      individual origin. */
+
+	if (inp == NULL) return GGI_EARGINVAL;
+
+	if (_gii_threadsafe) {
+		ggLock(inp->mutex);
+		ggLock(inp->queue->mutex);
+	}
+
+	if (inp->next == inp) return GGI_ENOTFOUND;
+
+	idx = inp;
+	do {
+		if ((origin == idx->origin) || 
+		    (origin == GII_EV_ORIGIN_NONE)) {
+			if (inp == idx) {
+
+				_giiEvQueueAllocate(inp);
+				inp->safequeue = NULL;
+				if (inp->cache != NULL) {
+					inp->cache->count--;
+					if (inp->cache->count == 0) 
+						_giiCacheFree(inp->cache);
+					inp->cache = _giiCacheAlloc();
+					inp->cache->count++;
+				}
+
+				if (_gii_threadsafe) {
+					ggUnlock(inp->next->queue->mutex);
+					ggLock(inp->next->mutex);
+					if (inp->next != inp->prev)
+						ggLock(inp->prev->mutex);
+				}
+				inp->next->prev = inp->prev;
+				inp->prev->next = inp->next;
+				*newhand = inp->next;
+				if (_gii_threadsafe) {
+					if (inp->next != inp->prev)
+						ggUnlock(inp->prev->mutex);
+					ggUnlock(inp->next->mutex);
+				}
+				inp->next = inp;
+				inp->prev = inp;
+				if (_gii_threadsafe) ggUnlock(inp->mutex);
+				/* newhand may still be a joined input,
+				 * so we inform the user of that. */
+				return 1;
+			}
+			if (_gii_threadsafe) {
+				ggLock(idx->mutex);
+				if (idx->next != inp)
+					ggLock(inp->next->mutex);
+				if (idx->prev != inp &&
+				    idx->next != idx->prev)
+					ggLock(inp->prev->mutex);
+			}
+			_giiEvQueueAllocate(idx);
+			inp->safequeue = NULL;
+			if (idx->cache != NULL) {
+				idx->cache->count--;
+				if (idx->cache->count == 0) 
+					_giiCacheFree(idx->cache);
+				idx->cache = _giiCacheAlloc();
+				idx->cache->count++;
+			}
+			idx->prev->next = idx->next;
+			idx->next->prev = idx->prev;
+			if (_gii_threadsafe) {
+				if (idx->prev != inp &&
+				    idx->next != idx->prev)
+					ggUnlock(inp->prev->mutex);
+				if (idx->next != inp)
+					ggUnlock(inp->next->mutex);
+			}
+			*newhand = idx;
+			idx->next = idx;
+			idx->prev = idx;
+			if (_gii_threadsafe) {
+				ggUnlock(idx->mutex);
+				ggUnlock(inp->queue->mutex);
+				ggUnlock(inp->mutex);
+			}
+			return GGI_OK;
+		}
+		idx = idx->next;
+	} while (idx != inp);
+
+	if (_gii_threadsafe) {
+		ggUnlock(inp->queue->mutex);
+		ggUnlock(inp->mutex);
+	}
+
+	return GGI_ENOTFOUND;
+}
+
 int giiEventsQueued(struct gii_input *inp, gii_event_mask mask)
 {
 	int count = 0;
@@ -868,7 +991,7 @@ int giiEventSend(struct gii_input *inp, gii_event *event)
 	return _giiEvQueueAdd(inp, event);
 }
 
-void giiPanic(char *format, ...)
+void giiPanic(const char *format, ...)
 {
 	va_list ap;
 
@@ -898,7 +1021,8 @@ static gii_deviceinfo *giiFindDeviceInfo(gii_input_t inp, uint32 origin)
 	return NULL;
 }
 
-int	     giiQueryDeviceInfo(gii_input_t inp, uint32 origin, gii_cmddata_getdevinfo *info)
+int giiQueryDeviceInfo(gii_input_t inp, uint32 origin,
+		       gii_cmddata_getdevinfo *info)
 {
 	gii_deviceinfo *di=giiFindDeviceInfo(inp,origin);
 	if (di) {
@@ -907,7 +1031,8 @@ int	     giiQueryDeviceInfo(gii_input_t inp, uint32 origin, gii_cmddata_getdevin
 	return -1;
 }
 
-static gii_deviceinfo *giiFindDeviceInfoByNumber(gii_input_t inp, uint32 number, uint32 *origin)
+static gii_deviceinfo *
+giiFindDeviceInfoByNumber(gii_input_t inp, uint32 number, uint32 *origin)
 {
 	struct gii_input *curr = inp;
 	gii_deviceinfo   *di;
@@ -923,7 +1048,8 @@ static gii_deviceinfo *giiFindDeviceInfoByNumber(gii_input_t inp, uint32 number,
 	return NULL;
 }
 
-int	     giiQueryDeviceInfoByNumber(gii_input_t inp, uint32 number, uint32 *origin, gii_cmddata_getdevinfo *info)
+int giiQueryDeviceInfoByNumber(gii_input_t inp, uint32 number, uint32 *origin,
+			       gii_cmddata_getdevinfo *info)
 {
 	gii_deviceinfo *di=giiFindDeviceInfoByNumber(inp,number,origin);
 	if (di) {
@@ -932,7 +1058,8 @@ int	     giiQueryDeviceInfoByNumber(gii_input_t inp, uint32 number, uint32 *orig
 	return -1;
 }
 
-int	     giiQueryValInfo   (gii_input_t inp, uint32 origin, uint32 valnumber, gii_cmddata_getvalinfo *info)
+int giiQueryValInfo(gii_input_t inp, uint32 origin, uint32 valnumber,
+		    gii_cmddata_getvalinfo *info)
 {
 	gii_deviceinfo *di=giiFindDeviceInfo(inp,origin);
 	if (di) {
