@@ -1,9 +1,9 @@
-/* $Id: gg.h,v 1.15 1999/04/30 19:09:18 marcus Exp $
+/* $Id: gg.h,v 1.1.1.1 2001/05/12 23:00:13 cegger Exp $
 ******************************************************************************
   
    LibGG - API header file
   
-   Copyright (C) 1998 Marcus Sundberg		[marcus@ggi-project.org]
+   Copyright (C) 1998-1999 Marcus Sundberg	[marcus@ggi-project.org]
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -33,6 +33,8 @@
 #include <ggi/errors.h>
 #include <ggi/system.h>
 
+
+__BEGIN_DECLS
 /*
 ******************************************************************************
  Initialize/Deinitialize library
@@ -84,26 +86,21 @@ void ggFreeConfig(void *conf);
 const char *ggMatchConfig(const void *conf, const char *name,
 			  const char *version);
 
+int ggConfigExpandAlias(void *confhandle, const char *list_in,
+			char *list_out, size_t outmax);
+
 /*
 ******************************************************************************
  Dynamic module loading
 ******************************************************************************
 */
 
-#ifdef _WIN32
-typedef HMODULE gg_module;
-#else
 typedef void *gg_module;
-#endif
+
+#define GG_MODULE_NULL	NULL
 
 #define GG_MODULE_GLOBAL 1
 
-#ifdef _WIN32
-#define GG_MODULE_NULL NULL /* FIXME ??? */
-#else
-#define GG_MODULE_NULL NULL
-#endif 
-
 /* Dynamicly load file */
 gg_module ggLoadModule(const char *filename, int flags);
 
@@ -166,4 +163,6 @@ int ggUnlock(void *lock);
 
 int ggTryLock(void *lock);
 
+__END_DECLS
+
 #endif /* _GGI_GG_H */
