@@ -1,4 +1,4 @@
-/* $Id: xwin.h,v 1.4 1999/02/11 00:35:18 marcus Exp $
+/* $Id: xwin.h,v 1.1.1.1 2001/05/12 23:00:14 cegger Exp $
 ******************************************************************************
 
    Xwin inputlib - use an existing X window as a LibGII input source
@@ -32,6 +32,7 @@
 
 /* Returns non-zero if we should queue the expose event */
 typedef int (gii_inputxwin_exposefunc)(void *arg, int x, int y, int w, int h);
+typedef int (gii_inputxwin_resizefunc)(void *arg, int w, int h, gii_event *ev);
 
 typedef struct {
 	Display *disp;		/* Display to use */
@@ -45,6 +46,12 @@ typedef struct {
 	void	*exposearg;	/* This is the argument we pass to exposefunc*/
 	void	*gglock;	/* If this is non-null we use it to ggLock()
 				   in the polling function */
+	gii_inputxwin_resizefunc *resizefunc; /* If this is non-null we call
+				   the specified function to fill in a suitable
+				   event to send due to the resize if it returns
+				   0. If it returns non-0, we don't queue any
+				   event. */
+	void	*resizearg;	/* This is the argument we pass to resizefunc*/
 } gii_inputxwin_arg;
 
 /* Send this event to change parameters on the fly */
