@@ -1,9 +1,10 @@
-/* $Id: gii.h,v 1.22 1999/04/22 20:20:08 marcus Exp $
+/* $Id: gii.h,v 1.2 2001/06/17 08:34:30 cegger Exp $
 ******************************************************************************
 
    LibGII internals
  
    Copyright (C) 1998 Andreas Beck	[becka@ggi-project.org]
+   Copyright (C) 1999 Marcus Sundberg	[marcus@ggi-project.org]
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -28,34 +29,41 @@
 #ifndef _GGI_INTERNAL_GII_H
 #define _GGI_INTERNAL_GII_H
 
-#define _BUILDING_LIBGII
+#define _INTERNAL_LIBGII
 
 #include "config.h"
 #include <ggi/system.h>
 #include <ggi/events.h>
 
 typedef struct gii_input  *gii_input_t;
-typedef struct gii_filter *gii_filter_t;
 
 #include <ggi/gii.h>
-#include <ggi/internal/gii_debug.h>
 
 
 #define GII_MAXSUBLIBS	256
 #define GII_SUBLIBMASK	0x000000ff
 #define GII_MAINMASK	0xffffff00
 
+__BEGIN_DECLS
+
+/* Exported variables */
+#ifdef BUILDING_LIBGII
+extern int                   _gii_threadsafe;
+extern void                 *_gii_global_lock;
+#else
+IMPORTVAR int                _gii_threadsafe;
+IMPORTVAR void              *_gii_global_lock;
+#endif
+
 /* Global variables */
 extern void                 *_giiconfhandle;
 extern void                 *_gii_safe_lock;
-extern int                   _gii_threadsafe;
+
+__END_DECLS
 
 /* Function types */
 typedef int		(giifunc_handler)(struct gii_input *inp,
 					  gii_event *event);
-typedef int		(giifunc_filterclose)(struct gii_filter *filt);
-typedef int		(giifunc_filterinit)(struct gii_filter *filt,
-					     const char *args, void *argptr);
 
 typedef gii_event_mask	(giifunc_eventpoll)(struct gii_input *inp, void *arg);
 typedef int		(giifunc_sendevent)(struct gii_input *inp,
@@ -97,7 +105,6 @@ typedef struct gii_input {
 	struct gii_input        *next,*prev;	/* Ring structure */
 
 	struct gii_dlhandle	*dlhand;	/* for closing */
-	struct gii_filter	*filters;	/* List of filters */
 
 	uint32			origin;
 	int 			maxfd;
@@ -124,8 +131,6 @@ typedef struct gii_input {
 
 #define GII_FLAGS_HASPOLLED	0x00000001
 
-#define GII_EV_TIMESTAMP(ev)  { gettimeofday(& (ev)->any.time, NULL); }
-
 #define GII_QUEUE_SIZE      8192
 #define GII_Q_THRESHOLD     (GII_QUEUE_SIZE - sizeof(gii_event) - 1)
 
@@ -157,8 +162,10 @@ typedef struct gii_dlhandle {
 	void		*init;		/* Init functions */
 } gii_dlhandle;
 
+
 /* Internal API calls
  */
+__BEGIN_DECLS
 
 gii_dlhandle *_giiLoadDL(const char *name, const char *version);
 int _giiCloseDL(gii_dlhandle *hand);
@@ -175,4 +182,8 @@ void _giiUpdateCache(gii_input *inp);
 int _giiSafeAdd(struct gii_input *inp, gii_event *ev);
 int _giiSafeMove(struct gii_input *toinp, struct gii_input *frominp);
 
+void _giiAsyncNotify(gii_input *inp);
+
+__END_DECLS
+
 #endif /* _GGI_INTERNAL_GII_H */
