@@ -1,9 +1,10 @@
-/* $Id: input.c,v 1.37 1999/05/16 15:48:41 marcus Exp $
+/* $Id: input.c,v 1.1.1.1 2001/05/12 23:00:22 cegger Exp $
 ******************************************************************************
 
    Linux_kbd: input via linux MEDIUMRAW mode.
 
-   Copyright (C) 1998-1999  Andrew Apted   [andrew@ggi-project.org]
+   Copyright (C) 1998-1999  Andrew Apted	[andrew@ggi-project.org]
+   Copyright (C) 1999       Marcus Sundberg	[marcus@ggi-project.org]
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -107,7 +108,7 @@ static inline int
 GII_keyboard_init(gii_input *inp, char *filename)
 {
 	int fd;
-	struct termios new;
+	struct termios tio;
 	linkbd_priv *priv;
 	void (*oldinhandler)(int);
 	void (*oldouthandler)(int);
@@ -136,20 +137,20 @@ GII_keyboard_init(gii_input *inp, char *filename)
 		perror("Linux-kbd: tcgetattr failed");
 	}
 
-	new = priv->old_termios;
+	tio = priv->old_termios;
 
-	new.c_lflag &= ~(ICANON | ECHO  | ISIG);
-	new.c_iflag &= ~(ISTRIP | IGNCR | ICRNL | INLCR | IXOFF | IXON);
-	new.c_iflag |= IGNBRK;
-	new.c_cc[VMIN]  = 0;
-	new.c_cc[VTIME] = 0;
+	tio.c_lflag &= ~(ICANON | ECHO  | ISIG);
+	tio.c_iflag &= ~(ISTRIP | IGNCR | ICRNL | INLCR | IXOFF | IXON);
+	tio.c_iflag |= IGNBRK;
+	tio.c_cc[VMIN]  = 0;
+	tio.c_cc[VTIME] = 0;
 
 	GIIDPRINT_MISC("Linux-kbd: calling tcsetattr()\n");
 
 	got_stopped = 0;
 	oldinhandler  = signal(SIGTTIN, sighandler);
 	oldouthandler = signal(SIGTTOU, sighandler);
-	if (tcsetattr(fd, TCSANOW, &new) < 0) {
+	if (tcsetattr(fd, TCSANOW, &tio) < 0) {
 		perror("Linux-kbd: tcsetattr failed");
 	}
 	signal(SIGTTIN, oldinhandler);
@@ -374,6 +375,7 @@ GII_keyboard_handle_data(gii_input *inp, int code)
 		ev.key.type = evKeyRepeat;
 	}
 	ev.key.button = code;
+	/* First update modifiers here so linkey.c can use the value */
 	ev.key.modifiers = priv->modifiers;
 
 	if (ev.key.type == evKeyRelease &&
@@ -447,6 +449,9 @@ GII_keyboard_handle_data(gii_input *inp, int code)
 
 		handle_modifier(priv, &ev);
 	}
+	/* Now update modifiers again to get the value _after_ the current
+	   event */
+	ev.key.modifiers = priv->modifiers;
 
 	if (ev.any.type == evKeyPress) {
 		priv->keydown_sym[code]    = ev.key.sym;
