@@ -1,4 +1,4 @@
-/* $Id: linkey.c,v 1.11 1999/05/16 18:24:06 marcus Exp $
+/* $Id: linkey.c,v 1.1.1.1 2001/05/12 23:00:22 cegger Exp $
 ******************************************************************************
 
    Convert Linux keysyms into LibGII syms and labels
@@ -44,7 +44,7 @@ static uint32 basic_trans(int sym, int islabel, uint32 *modifiers,
 	switch (typ) {
 	case KT_LATIN: 
 	case KT_LETTER: 
-		if (islabel && (val == 0x7f || val == 0x08)) {
+		if (val == 0x7f || val == 0x08) {
 			/* Backspace and delete mapping is a mess under Linux.
 			   We simply use the keycode to make it right.
 			*/
