@@ -1,4 +1,4 @@
-/* $Id: input.c,v 1.20 1999/05/13 16:15:23 marcus Exp $
+/* $Id: input.c,v 1.2 2001/06/17 08:34:30 cegger Exp $
 ******************************************************************************
 
    Mouse: input
@@ -27,6 +27,7 @@
 */
 
 #include <ggi/internal/gii-dl.h>
+#include <ggi/internal/gii_debug.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -56,6 +57,8 @@
 
 #define MAX_OPTLEN	255
 #define MAX_MOUSE_NAMES		8
+#define MAX_LINELEN     2048
+
 
 typedef struct {
 	/* Synonyms */
@@ -102,6 +105,7 @@ static MouseType mice_types[] =
     {{ "PS/2", "ps2", NULL }, "ps2", -1, 0 },
     {{ "mmanps2", "MouseManPlusPS/2", NULL }, "mmanps2", -1, 0 },
     {{ "imps2", "IMPS/2", NULL }, "imps2", -1, 0 },
+    {{ "lnxusb", "LinuxUSB", NULL }, "lnxusb", -1, 0 },
 
     /* Terminator */
     {{ NULL }, NULL, -1, 0 }
@@ -155,6 +159,7 @@ static int do_mouse_open(gii_input *inp, char *filename,
 			 int dtr, int rts, int baud)
 {
 	l_mouse_priv *mhook = L_MOUSE_PRIV(inp);
+	int ispipe = 0;
 
 	mhook->readonly = 0;
 	mhook->fd = open(filename, O_RDWR | O_NOCTTY | O_NONBLOCK);
@@ -173,26 +178,37 @@ static int do_mouse_open(gii_input *inp, char *filename,
 	GIIDPRINT_MISC("linux-mouse: Opened mouse file '%s' %s.\n",
 		       filename, mhook->readonly ? "ReadOnly" : "Read/Write");
 
-	if (mhook->type->default_baud >= 0) {
+#ifdef HAVE_LINUX_DEVICE_CHECK
+	do {
+		struct stat m_stat;
+			
+		if (fstat(mhook->fd, &m_stat) == 0 &&
+		    S_ISFIFO(m_stat.st_mode)) {
+			ispipe = 1;
+		}
+	} while (0);
+#endif
+
+	if (!ispipe && mhook->type->default_baud >= 0) {
 		int dowarn = 0;
 
 		/* Set up the termios state and baud rate */
 		tcflush(mhook->fd, TCIOFLUSH);
 		if (tcgetattr(mhook->fd, &mhook->old_termios) == 0) {
-			struct termios new = mhook->old_termios;
+			struct termios tio = mhook->old_termios;
 
 			if (baud < 0) {
 				baud = mhook->type->default_baud;
 			}
 
-			new.c_cflag = mhook->type->cflag | baud;
-			new.c_iflag = IGNBRK;
-			new.c_oflag = 0;
-			new.c_lflag = 0;
-			new.c_cc[VMIN]  = 1;
-			new.c_cc[VTIME] = 0;
+			tio.c_cflag = mhook->type->cflag | baud;
+			tio.c_iflag = IGNBRK;
+			tio.c_oflag = 0;
+			tio.c_lflag = 0;
+			tio.c_cc[VMIN]  = 1;
+			tio.c_cc[VTIME] = 0;
 
-			if (tcsetattr(mhook->fd, TCSANOW, &new) == 0) {
+			if (tcsetattr(mhook->fd, TCSANOW, &tio) == 0) {
 				mhook->have_old_termios = 1;
 			} else {
 				dowarn = 1;
@@ -203,6 +219,7 @@ static int do_mouse_open(gii_input *inp, char *filename,
 
 		/* Set up RTS and DTR modem lines */
 		if ((dtr >= 0) || (rts >= 0)) {
+#ifdef HAVE_TIOCMSET
 			unsigned int modem_lines;
 
 			if (ioctl(mhook->fd, TIOCMGET, &modem_lines) == 0) {
@@ -219,6 +236,12 @@ static int do_mouse_open(gii_input *inp, char *filename,
 			} else {
 				dowarn = 1;
 			}
+#else /* HAVE_TIOCMSET */
+			fprintf(stderr,
+				"input-linux-mouse: warning, this system does"
+				" not support TIOCMSET\n"
+				"        device may not work as expected\n");
+#endif /* HAVE_TIOCMSET */
 		}
 		if (dowarn) fprintf(stderr, serialfailstr);
 	}
@@ -265,7 +288,7 @@ static int get_from_file(char *fname, char *protname, char *mdev)
 	
 	fclose(fp);
 
-	return 0;
+	return (*protname=='\0');
 }
 
 
@@ -315,6 +338,86 @@ static void libvga_to_options(char *str, char *options)
 	}
 }
 
+#define COMMENT_CHAR '#'
+
+#define SKIPWHITE(str)  {while (isspace((int)*(str)) && *(str) != '\0' \
+				&& *(str) != COMMENT_CHAR) { (str)++; }}
+
+static char *get_value_from_XF86Config(char *str) 
+{
+	char *hlp;
+	SKIPWHITE(str);
+	if (*str=='"') {
+		hlp=++str;
+		while(*hlp!='"' &&*hlp!='"'&&*hlp!='"') {
+			hlp++;
+		}
+		if (*hlp=='"') *hlp=0;
+		return str;
+	} else {
+		hlp=str;
+		while (!isspace((int)*hlp) && *hlp != '\0' && *hlp != COMMENT_CHAR) 
+			hlp++;
+		if (isspace((int)*hlp)) *hlp=0;
+		return str;
+	}
+}
+
+static int get_from_XF86Config(const char *filename,
+				char *devname,char *protname, char *options)
+{
+	FILE *file;
+	char line[MAX_LINELEN];
+	int InPointerSection=0;
+	int GotKonfig=0;
+	int OptionCount=0;
+	
+	if (NULL==(file=fopen(filename,"r"))) return 1;
+	while(fgets(line, MAX_LINELEN, file) != NULL) {
+		char *lineptr=line;
+		SKIPWHITE(lineptr);
+		if (! InPointerSection) {
+			if (0==strncasecmp(lineptr,"Section",7)) {
+				lineptr+=7;
+				SKIPWHITE(lineptr);
+				if (0==strncasecmp(lineptr,"\"Pointer\"",9)) {
+					InPointerSection=1;
+				}
+			}
+		} else {
+			if (0==strncasecmp(lineptr,"EndSection",10)) {
+				break;
+			} else if (0==strncasecmp(lineptr,"Protocol",8)) {
+				lineptr+=8;
+				strncpy(protname,get_value_from_XF86Config(lineptr),MAX_OPTLEN);
+				GotKonfig=1;
+			} else if (0==strncasecmp(lineptr,"Device",6)) {
+				lineptr+=6;
+				strncpy(devname,get_value_from_XF86Config(lineptr),MAX_OPTLEN);
+			} else if (0==strncasecmp(lineptr,"BaudRate",8)) {
+				lineptr+=6;
+				strcat(options, "b");
+				strncat(options,get_value_from_XF86Config(lineptr),10);
+				OptionCount++;
+			} else if (0==strncasecmp(lineptr,"ClearRTS",8)) {
+				strcat(options, "r0");
+				OptionCount++;
+			} else if (0==strncasecmp(lineptr,"ClearDTR",8)) {
+				strcat(options, "d0");
+				OptionCount++;
+			} 
+			if (OptionCount>3) {
+				fprintf(stderr,"linux-mouse: More than 3 mouse options in XF86Config.\n"
+						"Parsing of bogus file aborted.\n");
+				GotKonfig=0;
+				break;
+			}
+		}
+	}
+	fclose(file);
+	return ! GotKonfig;
+}
+
 
 static void parse_mouse_specifier(char *spec, char *protname,
 				  char *devname, char *options)
@@ -348,8 +451,8 @@ static void parse_mouse_specifier(char *spec, char *protname,
 			char appendstr[] = "/input/linux-mouse";
 
 			dirname = ggGetUserDir();
-			if (strlen(dirname) + sizeof(appendstr) < 2040) {
-				sprintf(fname, "%s/.ggi%s", dirname,appendstr);
+			if (strlen(dirname) + sizeof(appendstr) < 2048) {
+				sprintf(fname, "%s%s", dirname, appendstr);
 				if (get_from_file(fname, protname, devname)
 				    == 0) {
 					return;
@@ -396,13 +499,19 @@ static void parse_mouse_specifier(char *spec, char *protname,
 			len = readlink(devname, buf, 1024);
 			if (len > 0 && len < 1024) {
 				buf[len] = '\0';
-				if (strncmp(devname, "/dev/gpm", 8) == 0) {
+				if (strstr(buf, "gpm") != NULL) {
 					strcpy(protname, "msc");
 					return;
 				}
 			}
 		} while (0);
 #endif
+
+		/* Try to parse XF86Config */
+		if (get_from_XF86Config("/etc/X11/XF86Config",
+					devname,protname,options)==0) return;
+		if (get_from_XF86Config("/etc/XF86Config",
+					devname,protname,options)==0) return;
 		
 		/* Try to parse SVGAlib config file as a last resort */
 		if (get_from_file("/etc/vga/libvga.config",
@@ -412,6 +521,7 @@ static void parse_mouse_specifier(char *spec, char *protname,
 			}
 			return;
 		}
+
 	}
 }
 
