@@ -1,4 +1,4 @@
-/* $Id: packetparse.c,v 1.15 1999/05/16 15:48:41 marcus Exp $
+/* $Id: packetparse.c,v 1.1.1.1 2001/05/12 23:00:27 cegger Exp $
 ******************************************************************************
 
    Mouse packet parsing
@@ -29,6 +29,8 @@
 #include "mouse.h"
 #include <string.h>
 #include <unistd.h>
+#include <errno.h>
+
 
 /**
  **  Event dispatching routines
@@ -126,9 +128,25 @@ static int parse_ms(gii_input *inp, uint8 *buf, int len)
 	dx = (sint8) (((buf[0] & 0x03) << 6) | (buf[1] & 0x3f));
 	dy = (sint8) (((buf[0] & 0x0c) << 4) | (buf[2] & 0x3f));
 
-	buttons = ((buf[0] & 0x10) >> 3)
-		| ((buf[0] & 0x20) >> 5);
+	/* Third button added by Christoph Egger
+	   (Christoph_Egger@t-online.de) */
+	if (buf[0] == 0x40 && !(mpriv->button_state|buf[1]|buf[2])) {
+		buttons = 4; /* third button on MS compatible mouse */
+	} else {
+		buttons = ((buf[0] & 0x10) >> 3)
+			| ((buf[0] & 0x20) >> 5);
+	}
 	
+	/* Should allow motion _and_ button change */
+	if ((dx == 0) && (dy == 0)
+	    && (buttons == (mpriv->button_state & ~4))) {
+		/* No move or change: toggle middle */
+		buttons = mpriv->button_state ^ 4;
+	} else {
+		/* Change: preserve middle */
+		buttons |= mpriv->button_state & 4;
+	}
+
 	mouse_send_movement(inp, dx, dy, 0, 0);
 	if (buttons != mpriv->button_state) {
 		mouse_send_buttons(inp, buttons, mpriv->button_state);
@@ -484,11 +502,11 @@ static int parse_imps2(gii_input *inp, uint8 *buf, int len)
 	mouse_priv *mpriv = MOUSE_PRIV(inp);
 
 	if ((buf[0] & 0xc0) != 0) {
-		GIIDPRINT_EVENTS("Invalid IntelliMouse PS/2 packet\n");
+		GIIDPRINT_EVENTS("Invalid IMPS/2 packet\n");
 		return 1;
 	}
 
-	buttons = buf[0] & 0x07;
+	buttons = buf[0] & 0x0f;
 	dx = (buf[0] & 0x10) ? buf[1] - 256 : buf[1];
 	dy = (buf[0] & 0x20) ? -(buf[2] - 256) : -buf[2];
 	wheel = (sint8)buf[3];
@@ -499,9 +517,9 @@ static int parse_imps2(gii_input *inp, uint8 *buf, int len)
 		mpriv->button_state = buttons;
 	}
 
-	GIIDPRINT_EVENTS("Got IntelliMouse PS/2 packet\n");
+	GIIDPRINT_EVENTS("Got IMPS/2 packet\n");
 
-	return 3;
+	return 4;
 }
 
 
@@ -547,39 +565,65 @@ static uint8 initdata_mmanps2[] =
 /* Mouse parsers
 ******************************************************************************
 */
-parser_type _gii_mouse_parsers[] = {
-	{ { "null", NULL }, parse_null, 0,
-	  NULL, 0, 0 },
-	{ { "ms", "Microsoft", NULL }, parse_ms, 3,
-	  NULL, 0, 0 },
-	{ { "msc", "MouseSystems", NULL }, parse_msc, 5,
-	  NULL, 0, 0 },
-	/* Logitech and MMSeries use the same protocol */
-	{ { "logi", "Logitech", "mm", "MMSeries", NULL }, parse_logi, 3,
-	  NULL, 0, 0 },
-	/* Sun and busmice use the same protocol */
-	{ { "sun", "bm", "BusMouse", NULL }, parse_sun, 3,
-	  NULL, 0, 0 },
-	{ { "mman", "MouseMan", NULL }, parse_mman, 3,
-	  NULL, 0, 0 },
-	/* Serial IntelliMouse or MouseMan+ */
-	{ { "ms3", "IntelliMouse", "mman+", NULL }, parse_ms3, 3,
-	  NULL, 0, 0 },
 
-	{ { "ps2", "PS/2", NULL }, parse_ps2, 3,
-	  initdata_ps2, sizeof(initdata_ps2), 0 },
-	/* PS/2 MouseMan+ */
-	{ { "mmanps2", "MouseManPlusPS/2", NULL }, parse_mmanps2, 3,
-	  initdata_mmanps2, sizeof(initdata_mmanps2), 1 },
-	/* PS/2 IntelliMouse */
-	{ { "imps2", "IMPS/2", NULL }, parse_imps2, 4,
-	  initdata_imps2 , sizeof(initdata_imps2), 1 },
+static parser_type pp_null =
+{ { "null", NULL }, parse_null, 0,
+  NULL, 0, 0, NULL };
+static parser_type pp_ms =
+{ { "ms", "Microsoft", NULL }, parse_ms, 3,
+  NULL, 0, 0, NULL };
+static parser_type pp_msc =
+{ { "msc", "MouseSystems", NULL }, parse_msc, 5,
+  NULL, 0, 0, NULL };
+/* Logitech and MMSeries use the same protocol */
+static parser_type pp_logi =
+{ { "logi", "Logitech", "mm", "MMSeries", NULL }, parse_logi, 3,
+  NULL, 0, 0, NULL };
+/* Sun and busmice use the same protocol */
+static parser_type pp_sun =
+{ { "sun", "bm", "BusMouse", NULL }, parse_sun, 3,
+  NULL, 0, 0, NULL };
+static parser_type pp_mman =
+{ { "mman", "MouseMan", NULL }, parse_mman, 3,
+  NULL, 0, 0, NULL };
+/* Serial IntelliMouse or MouseMan+ */
+static parser_type pp_ms3 =
+{ { "ms3", "IntelliMouse", "mman+", NULL }, parse_ms3, 3,
+  NULL, 0, 0, NULL };
 
-	/* Terminator */
-	{ { NULL }, NULL, -1, NULL, 0, 0 }
+static parser_type pp_ps2 =
+{ { "ps2", "PS/2", NULL }, parse_ps2, 3,
+  initdata_ps2, sizeof(initdata_ps2), GII_MIT_DONTCARE, NULL };
+/* PS/2 MouseMan+ */
+static parser_type pp_mmanps2 =
+{ { "mmanps2", "MouseManPlusPS/2", NULL }, parse_mmanps2, 3,
+  initdata_mmanps2, sizeof(initdata_mmanps2), GII_MIT_FALLBACK, &pp_ps2 };
+/* PS/2 IntelliMouse */
+static parser_type pp_imps2 =
+{ { "imps2", "IMPS/2", NULL }, parse_imps2, 4,
+  initdata_imps2 , sizeof(initdata_imps2), GII_MIT_FALLBACK, &pp_ps2 };
+/* Linux USB mouse - imps2 protocol without init string. */
+static parser_type pp_lnxusb =
+{ { "lnxusb", "LinuxUSB", NULL }, parse_imps2, 4,
+  NULL, 0, 0, NULL };
+
+parser_type *_gii_mouse_parsers[] = {
+	&pp_null,
+	&pp_ms,
+	&pp_msc,
+	&pp_logi,
+	&pp_sun,
+	&pp_mman,
+	&pp_ms3,
+
+	&pp_ps2,
+	&pp_mmanps2,
+	&pp_imps2,
+	&pp_lnxusb,
+
+	NULL	/* Terminator */
 };
 
-
 static int do_parse_packet(gii_input *inp)
 {
 	mouse_priv *mpriv = MOUSE_PRIV(inp);
@@ -655,6 +699,7 @@ gii_event_mask GII_mouse_poll(gii_input *inp, void *arg)
 			return 0;
 		}
 		/* Error, we try again next time */
+		if (errno == EAGAIN) return 0; /* Don't print warning here */
 		perror("Mouse: Error reading mouse");
 		return 0;
 	}
