@@ -1,4 +1,4 @@
-/* $Id: input.c,v 1.22 1999/05/09 14:01:48 marcus Exp $
+/* $Id: input.c,v 1.2 2001/06/17 08:34:31 cegger Exp $
 ******************************************************************************
 
    Xwin inputlib - use an existing X window as a LibGII input source
@@ -30,6 +30,7 @@
 #include <X11/Xlib.h>
 
 #include <ggi/internal/gii-dl.h>
+#include <ggi/internal/gii_debug.h>
 #include <ggi/input/xwin.h>
 #include "../x/xev.h"
 
@@ -53,6 +54,8 @@ typedef struct {
 	uint32	 relptr_keymask;
 	gii_inputxwin_exposefunc *exposefunc;
 	void	*exposearg;
+	gii_inputxwin_resizefunc *resizefunc;
+	void	*resizearg;
 	void	*lock;
 } xwin_priv;
 
@@ -77,22 +80,35 @@ static inline Cursor make_cursor(Display *disp, Window win)
 	return mycrsr;
 }
 
-static inline void handle_relptr(xwin_priv *priv)
+
+static void do_grab(xwin_priv *priv)
+{
+	XGrabPointer(priv->disp, priv->win, 1, 0,
+		     GrabModeAsync, GrabModeAsync,
+		     priv->win, priv->cursor, CurrentTime);
+	XWarpPointer(priv->disp, None, priv->win, 0, 0, 0, 0,
+		     priv->width/2, priv->height/2);
+	priv->relptr = 1;
+	priv->oldx = priv->width/2;
+	priv->oldy = priv->height/2;
+	GIIDPRINT_EVENTS("GII_xwin: Using relative pointerevents\n");
+}
+
+
+static void do_ungrab(xwin_priv *priv)
+{
+	XUngrabPointer(priv->disp, CurrentTime);
+	priv->relptr = 0;
+	GIIDPRINT_EVENTS("GII_xwin: Using absolute pointerevents\n");
+}
+
+
+static void handle_relptr(xwin_priv *priv)
 {
 	if (priv->relptr) {
-		XUngrabPointer(priv->disp, CurrentTime);
-		priv->relptr = 0;
-		GIIDPRINT_EVENTS("GII_xwin_eventpoll: Using absolute pointerevents\n");
+		do_ungrab(priv);
 	} else {
-		XGrabPointer(priv->disp, priv->win, 1, 0,
-			     GrabModeAsync, GrabModeAsync,
-			     priv->win, priv->cursor, CurrentTime);
-		XWarpPointer(priv->disp, None, priv->win, 0, 0, 0, 0,
-			     priv->width/2, priv->height/2);
-		priv->relptr = 1;
-		priv->oldx = priv->width/2;
-		priv->oldy = priv->height/2;
-		GIIDPRINT_EVENTS("GII_xwin_eventpoll: Using relative pointerevents\n");
+		do_grab(priv);
 	}
 }
 
@@ -158,10 +174,9 @@ static gii_event_mask GII_xwin_eventpoll(gii_input *inp, void *arg)
 	GIIDPRINT_EVENTS("GII_xwin_eventpoll(%p) called\n", inp);
 
 	if (priv->lock) ggLock(priv->lock);
-	
-	n = XEventsQueued(priv->disp, QueuedAfterReading);
 
-	while (n) {
+	n = 0;
+	while (n || (n = XEventsQueued(priv->disp, QueuedAfterReading))) {
 		XEvent xev;
 		gii_event giiev;
 		unsigned int keycode;
@@ -286,11 +301,11 @@ static gii_event_mask GII_xwin_eventpoll(gii_input *inp, void *arg)
 						xev.xmotion.y - priv->oldy;
 					realmove = 1;
 #undef ABS
-#define ABS(a) (((a)<0) ? -(a) : (a) )
+#define ABS(a) (((int)(a)<0) ? -(a) : (a) )
 					if (ABS(priv->width/2 - xev.xmotion.x)
-					    > 3 * priv->width / 8 ||
+					    > priv->width / 4 ||
 					    ABS(priv->height/2 - xev.xmotion.y)
-					    > 3 * priv->height / 8) {
+					    > priv->height / 4) {
 						center_pointer(priv);
 					}
 #undef ABS			
@@ -343,6 +358,19 @@ static gii_event_mask GII_xwin_eventpoll(gii_input *inp, void *arg)
 			giiev.expose.h = xev.xexpose.height;
 			break;
 
+		case ResizeRequest:
+			GIIDPRINT_EVENTS("GII_xwin_eventpoll: Resize\n");
+			giiev.any.origin = inp->origin;
+			if (priv->resizefunc != NULL) {
+				if (priv->resizefunc(priv->resizearg,
+						       xev.xresizerequest.width,
+						       xev.xresizerequest.height,
+						       &giiev)) {
+					goto dont_queue_this_event;
+				}
+			}
+			break;
+
 		default: 
 			GIIDPRINT_EVENTS("GII_xwin_eventpoll: Other Event (%d)\n",
 				      xev.type);
@@ -363,6 +391,8 @@ static gii_event_mask GII_xwin_eventpoll(gii_input *inp, void *arg)
 		   Well, this makes GCC happy at least... */
 		while(0){};
 	}
+	ggUnlock(priv->lock);
+
 	if (havecached) {
 		_giiEvQueueAdd(inp, &releasecache);
 		rc |= emKeyRelease;
@@ -370,7 +400,6 @@ static gii_event_mask GII_xwin_eventpoll(gii_input *inp, void *arg)
 			priv->symstat[releasecache.key.label] = 0;
 		}
 	}
-	ggUnlock(priv->lock);
 
 	return rc;
 }
@@ -444,7 +473,7 @@ static void send_devinfo(gii_input *inp, int dev)
 	ev.any.origin = inp->origin + dev;
 	ev.cmd.code   = GII_CMDCODE_GETDEVINFO;
 
-	dinfo = (gii_cmddata_getdevinfo *) &ev.cmd.data;
+	dinfo = (gii_cmddata_getdevinfo *) ev.cmd.data;
 	switch (dev) {
 	case 1:
 		*dinfo = mouse_devinfo;
@@ -496,6 +525,14 @@ GIIsendevent(gii_input *inp, gii_event *ev)
 		
 		update_winparam(priv);
 
+		return 0;
+	} else if (ev->cmd.code == GII_CMDCODE_PREFER_ABSPTR) {
+		xwin_priv  *priv = XWIN_PRIV(inp);
+		if (priv->relptr) do_ungrab(priv);
+		return 0;
+	} else if (ev->cmd.code == GII_CMDCODE_PREFER_RELPTR) {
+		xwin_priv  *priv = XWIN_PRIV(inp);
+		if (!priv->relptr) do_grab(priv);
 		return 0;
 	}
 
@@ -529,12 +566,14 @@ int GIIdlinit(gii_input *inp, const char *args, void *argptr)
 	priv->oldcode = 0;
 	memset(priv->symstat, 0, sizeof(priv->symstat));
 
-	priv->alwaysrel = xwinarg->ptralwaysrel;
-	priv->relptr = 0;
-	priv->relptr_keymask = RELPTR_KEYINUSE;
-	priv->exposefunc = xwinarg->exposefunc;
-	priv->exposearg = xwinarg->exposearg;
-	priv->lock = xwinarg->gglock;
+	priv->alwaysrel		= xwinarg->ptralwaysrel;
+	priv->relptr		= 0;
+	priv->relptr_keymask	= RELPTR_KEYINUSE;
+	priv->exposefunc	= xwinarg->exposefunc;
+	priv->exposearg		= xwinarg->exposearg;
+	priv->resizefunc	= xwinarg->resizefunc;
+	priv->resizearg		= xwinarg->resizearg;
+	priv->lock		= xwinarg->gglock;
 	
 	if (!xwinarg->wait) {
 		update_winparam(priv);
